import pino from 'pino'
import fs from 'fs'
import { getRootPath } from './storage'

// Array of log types
const LOG_TYPES = ['http', 'game', 'gameMaster', 'app', 'queue', 'command'] as const
type LogType = (typeof LOG_TYPES)[number]

const createLogger = (name: LogType) =>
  pino({
    level: process.env.LOG_LEVEL || 'debug',
    transport: {
      target: 'pino/file',
      options: {
        destination: `./logs/${name}.log`,
        mkdir: true
      }
    },
    formatters: {
      level: (label) => {
        return { level: label.toUpperCase() }
      }
    },
    timestamp: pino.stdTimeFunctions.isoTime,
    base: null
  })

// Create logger instances
const loggers = Object.fromEntries(LOG_TYPES.map((type) => [type, createLogger(type)])) as Record<LogType, ReturnType<typeof createLogger>>

const watchLogs = () => {
  LOG_TYPES.forEach((type) => {
    const logFile = getRootPath('logs', `${type}.log`)

    // Get initial file size
    const stats = fs.statSync(logFile)
    let lastPosition = stats.size

    fs.watch(logFile, (eventType) => {
      if (eventType !== 'change') {
        return
      }

      fs.stat(logFile, (err, stats) => {
        if (err) return

        if (stats.size > lastPosition) {
          const stream = fs.createReadStream(logFile, {
            start: lastPosition,
            end: stats.size
          })

          stream.on('data', (chunk) => {
            console.log(`[${type}]\n${chunk.toString()}`)
          })

          lastPosition = stats.size
        }
      })
    })
  })
}

export const { http: httpLogger, game: gameLogger, gameMaster: gameMasterLogger, app: appLogger, queue: queueLogger, command: commandLogger } = loggers

export { watchLogs }
